==> gravestones-1.2.0%2B1.21.8%2BA-sources.jar: net/pneumono/gravestones/block/AestheticGravestoneBlockEntity.java <==
package net.pneumono.gravestones.block;

import com.mojang.brigadier.exceptions.CommandSyntaxException;
import net.minecraft.class_11368;
import net.minecraft.class_11372;
import net.minecraft.class_1264;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2165;
import net.minecraft.class_2168;
import net.minecraft.class_2170;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_241;
import net.minecraft.class_243;
import net.minecraft.class_2558;
import net.minecraft.class_2561;
import net.minecraft.class_2564;
import net.minecraft.class_2583;
import net.minecraft.class_2680;
import net.minecraft.class_2741;
import net.minecraft.class_3218;
import net.minecraft.class_5712;
import net.minecraft.class_5837;
import net.minecraft.class_8242;
import net.minecraft.class_9288;
import net.minecraft.class_9323;
import net.minecraft.class_9334;
import net.minecraft.class_9473;
import net.pneumono.gravestones.Gravestones;
import net.pneumono.gravestones.content.GravestonesRegistry;
import net.pneumono.pneumonocore.util.MultiVersionUtil;
import org.jetbrains.annotations.Nullable;

import java.util.List;
import java.util.Objects;
import java.util.UUID;
import java.util.function.UnaryOperator;

public class AestheticGravestoneBlockEntity extends AbstractGravestoneBlockEntity {
    private class_1799 headStack = class_1799.field_8037;
    @Nullable
    private UUID editor;
    private class_8242 text = createText();
    private boolean waxed;

    public AestheticGravestoneBlockEntity(class_2338 pos, class_2680 state) {
        super(GravestonesRegistry.AESTHETIC_GRAVESTONE_ENTITY, pos, state);
    }

    protected class_8242 createText() {
        return new class_8242();
    }

    //? if >=1.21.8 {
    @Override
    public void method_11007(class_11372 view) {
        super.method_11007(view);
        if (!this.headStack.method_7960()) {
            view.method_71468("head", class_1799.field_24671, this.headStack);
        }
        view.method_71468("text", class_8242.field_43298, this.text);
        view.method_71472("is_waxed", this.waxed);
    }

    @Override
    public void method_11014(class_11368 view) {
        super.method_11014(view);
        this.headStack = view.method_71426("head", class_1799.field_24671).orElse(class_1799.field_8037);
        this.text = view.method_71426("text", class_8242.field_43298).map(this::parseLines).orElseGet(class_8242::new);
        this.waxed = view.method_71433("is_waxed", false);
    }
    //?} else if >=1.20.6 {
    /*@Override
    public void writeNbt(NbtCompound nbt, RegistryWrapper.WrapperLookup registries) {
        super.writeNbt(nbt, registries);
        RegistryOps<NbtElement> ops = registries.getOps(NbtOps.INSTANCE);

        if (!this.headStack.isEmpty()) {
            VersionUtil.put(ops, nbt, "head", ItemStack.CODEC, this.headStack);
        }
        VersionUtil.put(ops, nbt, "text", SignText.CODEC, this.text);
        nbt.putBoolean("is_waxed", this.waxed);
    }

    @Override
    public void readNbt(NbtCompound nbt, RegistryWrapper.WrapperLookup registries) {
        super.readNbt(nbt, registries);
        RegistryOps<NbtElement> ops = registries.getOps(NbtOps.INSTANCE);

        this.headStack = VersionUtil.get(ops, nbt, "head", ItemStack.CODEC).orElse(ItemStack.EMPTY);
        this.text = VersionUtil.get(ops, nbt, "text", SignText.CODEC).map(this::parseLines).orElseGet(SignText::new);
        this.waxed = nbt.getBoolean("is_waxed"/^? if >=1.21.5 {^//^, false^//^?}^/);
    }
    *///?} else {
    /*@Override
    public void writeNbt(NbtCompound nbt) {
        super.writeNbt(nbt);

        if (!this.headStack.isEmpty()) {
            VersionUtil.put(nbt, "head", ItemStack.CODEC, this.headStack);
        }
        VersionUtil.put(nbt, "text", SignText.CODEC, this.text);
        nbt.putBoolean("is_waxed", this.waxed);
    }

    @Override
    public void readNbt(NbtCompound nbt) {
        super.readNbt(nbt);

        this.headStack = VersionUtil.get(nbt, "head", ItemStack.CODEC).orElse(ItemStack.EMPTY);
        this.text = VersionUtil.get(nbt, "text", SignText.CODEC).map(this::parseLines).orElseGet(SignText::new);
        this.waxed = nbt.getBoolean("is_waxed"/^? if >=1.21.5 {^/, false/^?}^/);
    }
    *///?}

    //? if >=1.20.6 {
    @Override
    protected void method_57567(class_9323.class_9324 builder) {
        super.method_57567(builder);
        builder.method_57840(class_9334.field_49622, class_9288.method_57493(List.of(this.headStack)));
    }

    @Override
    protected void method_57568(class_9473 components) {
        super.method_57568(components);
        this.headStack = components.method_58695(class_9334.field_49622, class_9288.field_49334).method_58114();
    }
    //?}

    //? if >=1.21.8 {
    @SuppressWarnings("deprecation")
    @Override
    public void method_57569(class_11372 view) {
        super.method_57569(view);
        view.method_71478("head");
    }
    //?} else if >=1.20.6 {
    /*@SuppressWarnings("deprecation")
    @Override
    public void removeFromCopiedStackNbt(NbtCompound nbt) {
        super.removeFromCopiedStackNbt(nbt);
        nbt.remove("head");
    }
    *///?}

    private class_8242 parseLines(class_8242 signText) {
        for (int i = 0; i < 4; ++i) {
            class_2561 text = this.parseLine(signText.method_49859(i, false));
            class_2561 text2 = this.parseLine(signText.method_49859(i, true));
            signText = signText.method_49858(i, text, text2);
        }
        return signText;
    }

    private class_2561 parseLine(class_2561 text) {
        class_1937 world = this.field_11863;
        if (world instanceof class_3218 serverWorld) {
            try {
                return class_2564.method_10881(AestheticGravestoneBlockEntity.createCommandSource(null, serverWorld, this.field_11867), text, null, 0);
            } catch (CommandSyntaxException ignored) {}
        }
        return text;
    }

    public void tryChangeText(class_1657 player, List<class_5837> messages) {
        if (!this.isWaxed() && player.method_5667().equals(this.getEditor()) && this.field_11863 != null) {
            this.changeText(text -> this.getTextWithMessages(player, messages, text));
            this.setEditor(null);
            this.field_11863.method_8413(this.method_11016(), this.method_11010(), this.method_11010(), class_2248.field_31036);
        } else {
            Gravestones.LOGGER.warn("Player {} just tried to change non-editable gravestone", player.method_5477().getString());
        }
    }

    public boolean changeText(UnaryOperator<class_8242> textChanger) {
        return this.setText(textChanger.apply(this.getText()));
    }

    private class_8242 getTextWithMessages(class_1657 player, List<class_5837> messages, class_8242 text) {
        for (int i = 0; i < messages.size(); i++) {
            class_5837 filteredMessage = messages.get(i);
            class_2583 style = text.method_49859(i, player.method_33793()).method_10866();
            if (player.method_33793()) {
                text = text.method_49857(i, class_2561.method_43470(filteredMessage.method_45061()).method_10862(style));
            } else {
                text = text.method_49858(i, class_2561.method_43470(filteredMessage.comp_841()).method_10862(style), class_2561.method_43470(filteredMessage.method_45061()).method_10862(style));
            }
        }

        return text;
    }

    public boolean runCommandClickEvent(class_1657 player, class_1937 world, class_2338 pos) {
        boolean hasRunCommand = false;

        for (class_2561 text : this.getText().method_49877(player.method_33793())) {
            class_2583 style = text.method_10866();
            //? if >=1.21.5 {
            if (style.method_10970() instanceof class_2558.class_10609(String value)) {
            //?} else {
            /*ClickEvent clickEvent = style.getClickEvent();
            if (clickEvent != null && clickEvent.getAction() == ClickEvent.Action.RUN_COMMAND) {
                String value = clickEvent.getValue();
            *///?}
                class_2170 manager = Objects.requireNonNull(MultiVersionUtil.getWorld(player).method_8503()).method_3734();
                class_2168 commandSource = createCommandSource(player, world, pos);
                String command = value.startsWith("/") ? value.substring(1) : value;
                manager.method_9249(manager.method_9235().parse(command, commandSource), command);
                hasRunCommand = true;
            }
        }

        return hasRunCommand;
    }

    private static class_2168 createCommandSource(@Nullable class_1657 player, class_1937 world, class_2338 pos) {
        String string = player == null ? "Gravestone" : player.method_5477().getString();
        class_2561 text = player == null ? class_2561.method_43470("Gravestone") : player.method_5476();
        return new class_2168(class_2165.field_17395, class_243.method_24953(pos), class_241.field_1340, (class_3218)world, 2, string, text, world.method_8503(), player);
    }

    public boolean isPlayerTooFarToEdit(UUID uuid) {
        if (this.field_11863 == null) {
            return true;
        }
        class_1657 playerEntity = this.field_11863.method_18470(uuid);
        return playerEntity == null || /*? if >=1.20.6 {*/!playerEntity.method_56093(this.method_11016(), 4.0)/*?} else {*//*playerEntity.squaredDistanceTo(this.getPos().getX(), this.getPos().getY(), this.getPos().getZ()) > 64.0*//*?}*/;
    }

    @Override
    public class_2350 getGravestoneDirection() {
        class_1937 world = method_10997();
        if (world != null) {
            class_2680 state = world.method_8320(method_11016());
            if (state.method_28501().contains(class_2741.field_12481)) {
                return state.method_11654(class_2741.field_12481);
            }
        }
        return class_2350.field_11043;
    }

    public void setHeadStack(@Nullable class_1309 entity, class_1799 headStack) {
        //? if >=1.21.1 {
        this.headStack = headStack.method_60504(1, entity);
        //?} else {
        /*this.headStack = headStack.copyWithCount(1);
        if (!(entity instanceof PlayerEntity player) || !player.isCreative()) {
            headStack.decrement(1);
        }
        *///?}
        this.updateListeners();
        Objects.requireNonNull(this.method_10997()).method_43276(
                class_5712.field_28733, method_11016(), class_5712.class_7397.method_43286(entity, method_11010())
        );
    }

    public class_1799 getHeadStack() {
        return headStack;
    }

    //? if >=1.21.5 {
    @Override
    public void method_66473(class_2338 pos, class_2680 oldState) {
        if (this.field_11863 != null) {
            class_1264.method_5449(this.field_11863, pos.method_10263(), pos.method_10264(), pos.method_10260(), this.headStack);
        }
    }
    //?}

    public void setEditor(@Nullable UUID editor) {
        this.editor = editor;
    }

    @Nullable
    public UUID getEditor() {
        return this.editor;
    }

    public boolean setText(class_8242 text) {
        if (this.text != text) {
            this.text = text;
            this.updateListeners();
            return true;
        } else {
            return false;
        }
    }

    public class_8242 getText() {
        return this.text;
    }

    public boolean setWaxed(boolean waxed) {
        if (this.waxed != waxed) {
            this.waxed = waxed;
            this.updateListeners();
            return true;
        } else {
            return false;
        }
    }

    public boolean isWaxed() {
        return this.waxed;
    }

    private void updateListeners() {
        this.method_5431();
        Objects.requireNonNull(this.field_11863).method_8413(this.method_11016(), this.method_11010(), this.method_11010(), class_2248.field_31036);
    }
}

==> gravestones-1.2.1%2B1.21.5%2BA-sources.jar: net/pneumono/gravestones/block/AestheticGravestoneBlockEntity.java <==
package net.pneumono.gravestones.block;

import com.mojang.brigadier.exceptions.CommandSyntaxException;
import net.minecraft.class_1264;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2165;
import net.minecraft.class_2168;
import net.minecraft.class_2170;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_241;
import net.minecraft.class_243;
import net.minecraft.class_2487;
import net.minecraft.class_2509;
import net.minecraft.class_2520;
import net.minecraft.class_2558;
import net.minecraft.class_2561;
import net.minecraft.class_2564;
import net.minecraft.class_2583;
import net.minecraft.class_2680;
import net.minecraft.class_2741;
import net.minecraft.class_3218;
import net.minecraft.class_5712;
import net.minecraft.class_5837;
import net.minecraft.class_6903;
import net.minecraft.class_7225;
import net.minecraft.class_8242;
import net.minecraft.class_9288;
import net.minecraft.class_9323;
import net.minecraft.class_9334;
import net.minecraft.class_9473;
import net.pneumono.gravestones.Gravestones;
import net.pneumono.gravestones.content.GravestonesRegistry;
import net.pneumono.pneumonocore.util.MultiVersionUtil;
import org.jetbrains.annotations.Nullable;

import java.util.List;
import java.util.Objects;
import java.util.UUID;
import java.util.function.UnaryOperator;
import net.pneumono.gravestones.multiversion.VersionUtil;
//?} else {
/*import net.minecraft.nbt.NbtCompound;
import net.pneumono.gravestones.multiversion.VersionUtil;
*///?}

public class AestheticGravestoneBlockEntity extends AbstractGravestoneBlockEntity {
    private class_1799 headStack = class_1799.field_8037;
    @Nullable
    private UUID editor;
    private class_8242 text = createText();
    private boolean waxed;

    public AestheticGravestoneBlockEntity(class_2338 pos, class_2680 state) {
        super(GravestonesRegistry.AESTHETIC_GRAVESTONE_ENTITY, pos, state);
    }

    protected class_8242 createText() {
        return new class_8242();
    }

    //? if >=1.21.8 {
    /*@Override
    public void writeData(WriteView view) {
        super.writeData(view);
        if (!this.headStack.isEmpty()) {
            view.put("head", ItemStack.CODEC, this.headStack);
        }
        view.put("text", SignText.CODEC, this.text);
        view.putBoolean("is_waxed", this.waxed);
    }

    @Override
    public void readData(ReadView view) {
        super.readData(view);
        this.headStack = view.read("head", ItemStack.CODEC).orElse(ItemStack.EMPTY);
        this.text = view.read("text", SignText.CODEC).map(this::parseLines).orElseGet(SignText::new);
        this.waxed = view.getBoolean("is_waxed", false);
    }
    *///?} else if >=1.20.6 {
    @Override
    public void method_11007(class_2487 nbt, class_7225.class_7874 registries) {
        super.method_11007(nbt, registries);
        class_6903<class_2520> ops = registries.method_57093(class_2509.field_11560);

        if (!this.headStack.method_7960()) {
            VersionUtil.put(ops, nbt, "head", class_1799.field_24671, this.headStack);
        }
        VersionUtil.put(ops, nbt, "text", class_8242.field_43298, this.text);
        nbt.method_10556("is_waxed", this.waxed);
    }

    @Override
    public void method_11014(class_2487 nbt, class_7225.class_7874 registries) {
        super.method_11014(nbt, registries);
        class_6903<class_2520> ops = registries.method_57093(class_2509.field_11560);

        this.headStack = VersionUtil.get(ops, nbt, "head", class_1799.field_24671).orElse(class_1799.field_8037);
        this.text = VersionUtil.get(ops, nbt, "text", class_8242.field_43298).map(this::parseLines).orElseGet(class_8242::new);
        this.waxed = nbt.method_68566("is_waxed"/*? if >=1.21.5 {*/, false/*?}*/);
    }
    //?} else {
    /*@Override
    public void writeNbt(NbtCompound nbt) {
        super.writeNbt(nbt);

        if (!this.headStack.isEmpty()) {
            VersionUtil.put(nbt, "head", ItemStack.CODEC, this.headStack);
        }
        VersionUtil.put(nbt, "text", SignText.CODEC, this.text);
        nbt.putBoolean("is_waxed", this.waxed);
    }

    @Override
    public void readNbt(NbtCompound nbt) {
        super.readNbt(nbt);

        this.headStack = VersionUtil.get(nbt, "head", ItemStack.CODEC).orElse(ItemStack.EMPTY);
        this.text = VersionUtil.get(nbt, "text", SignText.CODEC).map(this::parseLines).orElseGet(SignText::new);
        this.waxed = nbt.getBoolean("is_waxed"/^? if >=1.21.5 {^/, false/^?}^/);
    }
    *///?}

    //? if >=1.20.6 {
    @Override
    protected void method_57567(class_9323.class_9324 builder) {
        super.method_57567(builder);
        builder.method_57840(class_9334.field_49622, class_9288.method_57493(List.of(this.headStack)));
    }

    @Override
    protected void method_57568(class_9473 components) {
        super.method_57568(components);
        this.headStack = components.method_58695(class_9334.field_49622, class_9288.field_49334).method_58114();
    }
    //?}

    //? if >=1.21.8 {
    /*@SuppressWarnings("deprecation")
    @Override
    public void removeFromCopiedStackData(WriteView view) {
        super.removeFromCopiedStackData(view);
        view.remove("head");
    }
    *///?} else if >=1.20.6 {
    @SuppressWarnings("deprecation")
    @Override
    public void method_57569(class_2487 nbt) {
        super.method_57569(nbt);
        nbt.method_10551("head");
    }
    //?}

    private class_8242 parseLines(class_8242 signText) {
        for (int i = 0; i < 4; ++i) {
            class_2561 text = this.parseLine(signText.method_49859(i, false));
            class_2561 text2 = this.parseLine(signText.method_49859(i, true));
            signText = signText.method_49858(i, text, text2);
        }
        return signText;
    }

    private class_2561 parseLine(class_2561 text) {
        class_1937 world = this.field_11863;
        if (world instanceof class_3218 serverWorld) {
            try {
                return class_2564.method_10881(AestheticGravestoneBlockEntity.createCommandSource(null, serverWorld, this.field_11867), text, null, 0);
            } catch (CommandSyntaxException ignored) {}
        }
        return text;
    }

    public void tryChangeText(class_1657 player, List<class_5837> messages) {
        if (!this.isWaxed() && player.method_5667().equals(this.getEditor()) && this.field_11863 != null) {
            this.changeText(text -> this.getTextWithMessages(player, messages, text));
            this.setEditor(null);
            this.field_11863.method_8413(this.method_11016(), this.method_11010(), this.method_11010(), class_2248.field_31036);
        } else {
            Gravestones.LOGGER.warn("Player {} just tried to change non-editable gravestone", player.method_5477().getString());
        }
    }

    public boolean changeText(UnaryOperator<class_8242> textChanger) {
        return this.setText(textChanger.apply(this.getText()));
    }

    private class_8242 getTextWithMessages(class_1657 player, List<class_5837> messages, class_8242 text) {
        for (int i = 0; i < messages.size(); i++) {
            class_5837 filteredMessage = messages.get(i);
            class_2583 style = text.method_49859(i, player.method_33793()).method_10866();
            if (player.method_33793()) {
                text = text.method_49857(i, class_2561.method_43470(filteredMessage.method_45061()).method_10862(style));
            } else {
                text = text.method_49858(i, class_2561.method_43470(filteredMessage.comp_841()).method_10862(style), class_2561.method_43470(filteredMessage.method_45061()).method_10862(style));
            }
        }

        return text;
    }

    public boolean runCommandClickEvent(class_1657 player, class_1937 world, class_2338 pos) {
        boolean hasRunCommand = false;

        for (class_2561 text : this.getText().method_49877(player.method_33793())) {
            class_2583 style = text.method_10866();
            //? if >=1.21.5 {
            if (style.method_10970() instanceof class_2558.class_10609(String value)) {
            //?} else {
            /*ClickEvent clickEvent = style.getClickEvent();
            if (clickEvent != null && clickEvent.getAction() == ClickEvent.Action.RUN_COMMAND) {
                String value = clickEvent.getValue();
            *///?}
                class_2170 manager = Objects.requireNonNull(MultiVersionUtil.getWorld(player).method_8503()).method_3734();
                class_2168 commandSource = createCommandSource(player, world, pos);
                String command = value.startsWith("/") ? value.substring(1) : value;
                manager.method_9249(manager.method_9235().parse(command, commandSource), command);
                hasRunCommand = true;
            }
        }

        return hasRunCommand;
    }

    private static class_2168 createCommandSource(@Nullable class_1657 player, class_1937 world, class_2338 pos) {
        String string = player == null ? "Gravestone" : player.method_5477().getString();
        class_2561 text = player == null ? class_2561.method_43470("Gravestone") : player.method_5476();
        return new class_2168(class_2165.field_17395, class_243.method_24953(pos), class_241.field_1340, (class_3218)world, 2, string, text, world.method_8503(), player);
    }

    public boolean isPlayerTooFarToEdit(UUID uuid) {
        if (this.field_11863 == null) {
            return true;
        }
        class_1657 playerEntity = this.field_11863.method_18470(uuid);
        return playerEntity == null || /*? if >=1.20.6 {*/!playerEntity.method_56093(this.method_11016(), 4.0)/*?} else {*//*playerEntity.squaredDistanceTo(this.getPos().getX(), this.getPos().getY(), this.getPos().getZ()) > 64.0*//*?}*/;
    }

    @Override
    public class_2350 getGravestoneDirection() {
        class_1937 world = method_10997();
        if (world != null) {
            class_2680 state = world.method_8320(method_11016());
            if (state.method_28501().contains(class_2741.field_12481)) {
                return state.method_11654(class_2741.field_12481);
            }
        }
        return class_2350.field_11043;
    }

    public void setHeadStack(@Nullable class_1309 entity, class_1799 headStack) {
        //? if >=1.21.1 {
        this.headStack = headStack.method_60504(1, entity);
        //?} else {
        /*this.headStack = headStack.copyWithCount(1);
        if (!(entity instanceof PlayerEntity player) || !player.isCreative()) {
            headStack.decrement(1);
        }
        *///?}
        this.updateListeners();
        Objects.requireNonNull(this.method_10997()).method_43276(
                class_5712.field_28733, method_11016(), class_5712.class_7397.method_43286(entity, method_11010())
        );
    }

    public class_1799 getHeadStack() {
        return headStack;
    }

    //? if >=1.21.5 {
    @Override
    public void method_66473(class_2338 pos, class_2680 oldState) {
        if (this.field_11863 != null) {
            class_1264.method_5449(this.field_11863, pos.method_10263(), pos.method_10264(), pos.method_10260(), this.headStack);
        }
    }
    //?}

    public void setEditor(@Nullable UUID editor) {
        this.editor = editor;
    }

    @Nullable
    public UUID getEditor() {
        return this.editor;
    }

    public boolean setText(class_8242 text) {
        if (this.text != text) {
            this.text = text;
            this.updateListeners();
            return true;
        } else {
            return false;
        }
    }

    public class_8242 getText() {
        return this.text;
    }

    public boolean setWaxed(boolean waxed) {
        if (this.waxed != waxed) {
            this.waxed = waxed;
            this.updateListeners();
            return true;
        } else {
            return false;
        }
    }

    public boolean isWaxed() {
        return this.waxed;
    }

    private void updateListeners() {
        this.method_5431();
        Objects.requireNonNull(this.field_11863).method_8413(this.method_11016(), this.method_11010(), this.method_11010(), class_2248.field_31036);
    }
}
